# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *
from collections import defaultdict, OrderedDict
from itertools import combinations
import re

def condense(ss_unescaped):
    """
    Given multiple strings, returns a compressed regular expression just
    for these strings

    >>> condense(['she', 'he', 'her', 'hemoglobin'])
    'he(moglobin|r)?|she'
    """
    def estimated_len(longg, short):
        return (3
                + len(short)
                + sum(map(len, longg))
                - len(longg)
                * (len(short) - 1)
                - 1 )

    def stupid_len(longg):
        return sum(map(len, longg)) + len(longg)

    ss = [re.escape(s) for s in set(ss_unescaped)]
    ss.sort(key=len)

    short2long = defaultdict(lambda: {'p':[],'s':[]})

    for short, longg in combinations(ss, 2):
        if longg.startswith(short):
            short2long[short]['p'].append(longg)
        if longg.endswith(short):
            short2long[short]['s'].append(longg)

    short2long = sorted(list(short2long.items()),
                        key=lambda x: len(x[0]),
                        reverse=True)

    output = []
    objs = set(ss)

    for s, pre_sur in short2long:
        pp = set(pre_sur['p']) & objs
        ss = set(pre_sur['s']) & objs
        if ((stupid_len(pp) - estimated_len(pp, s))
            < (stupid_len(ss) - estimated_len(ss, s))):
            reg = (r'({heads})?{surfix}'
                    .format(surfix=s,
                           heads='|'.join(sorted([p[:-len(s)] for p in ss],
                           key=len,
                           reverse=True))))
            assert len(reg) == estimated_len(ss, s)
            output.append(reg)
            objs -= (ss | set([s]))
        elif ((stupid_len(pp) - estimated_len(pp, s))
            > (stupid_len(ss) - estimated_len(ss, s))):
            reg = (r'{prefix}({tails})?'
                .format(prefix=s,
                        tails='|'.join(sorted([p[len(s):] for p in pp],
                        key=len,
                        reverse=True))))
            assert len(reg) == estimated_len(pp, s)
            output.append(reg)
            objs -= (pp | set([s]))

    for residual in objs:
        output.append(residual)
    return re.sub(r'\(([^)])\)\?', r'\1?', r'|'.join(output))

def is_solid(regex):
    """
    Check the given regular expression is solid.

    >>> is_solid(r'a')
    True
    >>> is_solid(r'[ab]')
    True
    >>> is_solid(r'(a|b|c)')
    True
    >>> is_solid(r'(a|b|c)?')
    True
    >>> is_solid(r'(a|b)(c)')
    False
    >>> is_solid(r'(a|b)(c)?')
    False
    """

    shape = re.sub(r'(\\.|[^\[\]\(\)\|\?\+\*])', '#', regex)
    skeleton = shape.replace('#', '')
    if len(shape) <= 1:
        return True
    if re.match(r'^\[[^\]]*\][\*\+\?]?$', shape):
        return True
    if re.match(r'^\([^\(]*\)[\*\+\?]?$', shape):
        return True
    if re.match(r'^\(\)#*?\)\)', skeleton):
        return True
    else:
        return False

def is_packed(regex):
    """
    Check if the regex is solid and packed into a pair of parens
    """
    return is_solid(regex) and regex[0] == '('

def consolidate(regex):
    """
    Put on a pair of parens (with no catch tag) outside the regex,
    if the regex is not yet consolidated
    """
    if is_solid(regex):
        return regex
    else:
        return '({})'.format(regex)

def danger_unpack(regex):
    """
    Remove the outermost parens

    >>> unpack(r'(abc)')
    'abc'
    >>> unpack(r'(?:abc)')
    'abc'
    >>> unpack(r'(?P<xyz>abc)')
    'abc'
    >>> unpack(r'[abc]')
    '[abc]'
    """

    if is_packed(regex):
        return re.sub(r'^\((\?(:|P<.*?>))?(?P<content>.*?)\)$', r'\g<content>', regex)
    else:
        return regex

def unpack(regex):
    """
    Remove the outermost parens, keep the (?P...) one

    >>> unpack(r'(abc)')
    'abc'
    >>> unpack(r'(?:abc)')
    'abc'
    >>> unpack(r'(?P<xyz>abc)')
    '(?P<xyz>abc)'
    >>> unpack(r'[abc]')
    '[abc]'
    """
    if is_packed(regex) and not regex.startswith('(?P<'):
        return re.sub(r'^\((\?:)?(?P<content>.*?)\)$', r'\g<content>', regex)
    else:
        return regex

def parallel(regex_list):
    """
    Join the given regexes using r'|'

    >>> parallel([r'abc', r'def'])
    'abc|def'
    >>> parallel([r'abc', r'd|ef'])
    'abc|def'
    >>> parallel([r'abc', r'(d|ef)'])
    'abc|d|ef'
    """
    
    return '|'.join([unpack(regex) for regex in regex_list])

def nocatch(regex):
    """
    Put on a pair of parens (with no catch tag) outside the regex,
    if the regex is not yet packed;
    modified the outmost parens by adding nocatch tag
    """
    if is_solid(regex) and not is_packed(regex):
        return regex
    else:
        return '(?:{})'.format(danger_unpack(regex))

def concat(regex_list):
    """
    Concat multiple regular expression into one, if the given regular expression is not packed,
    a pair of paren will be add.

    >>> reg_1 = r'a|b'
    >>> reg_2 = r'(c|d|e)'
    >>> concat([reg_1, reg2])
    (a|b)(c|d|e)
    """
    output_list = []

    for regex in regex_list:
        output_list.append(consolidate(regex))
    return r''.join(output_list)

def nocatchall(regex):
    """
    Return a regex with all parens has a no catch tag
    """
    return re.sub(r'(?<!\\)(?P<leading>(\\\\)*)\((\?(:|P<.*?>))?', r'\g<leading>(?:', regex)

def option(regex):
    """
    return a regex has a option tag

    >>> option(r'[ab]')
    '[ab]?'
    >>> option(r'(abc)')
    '(abc)?'
    >>> option('abc')
    '(abc)?'
    """

    return nocatch(regex) + '?'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

"""
test_retools
----------------------------------

Tests for `retools` module.
"""

import unittest
from txttk import retools
import re

class RetoolsTestCase(unittest.TestCase):
    def test_condense(self):
        words = ['hello', 'hellokitty', 'hellomonkey', 'goodbye', 'hell',
                 'he', 'his', 'hi', 'bye', 'history', 'story', 'condense',
                 'hematoma', 'lymphoma']
        regex = retools.condense(words)
        condensed = re.compile(regex)
        for word in words:
            self.assertTrue(condensed.match(word))

        negative_words = ['akkk', 'adadkjkl ', 'avxnbcjn']
        for word in negative_words:
            self.assertFalse(condensed.match(word))

    def test_is_solid(self):
        regex = r'a'
        self.assertTrue(retools.is_solid(regex))
        regex = r'[ab]'
        self.assertTrue(retools.is_solid(regex))
        regex = r'(a|b|c)'
        self.assertTrue(retools.is_solid(regex))
        regex = r'(a|b|c)?'
        self.assertTrue(retools.is_solid(regex))
        regex = r'(ab)c'
        self.assertFalse(retools.is_solid(regex))
        regex = r'(ab)c?'
        self.assertFalse(retools.is_solid(regex))
        regex = r'(a)(b|c)'
        self.assertFalse(retools.is_solid(regex))
        regex = r'(a)(b|c)?'
        self.assertFalse(retools.is_solid(regex))

    def test_is_packed(self):
        regex = r'a'
        self.assertFalse(retools.is_packed(regex))
        regex = r'[ab]'
        self.assertFalse(retools.is_packed(regex))
        regex = r'(a|b|c)'
        self.assertTrue(retools.is_packed(regex))
        regex = r'(ab)c'
        self.assertFalse(retools.is_packed(regex))

    def test_consolidate(self):
        regex = r'a|b'
        wanted = r'(a|b)'
        result = retools.consolidate(regex)
        self.assertEqual(result, wanted)

        regex = r'[ab]'
        wanted = r'[ab]'
        result = retools.consolidate(regex)
        self.assertEqual(result, wanted)

        regex = r'(?:a|b)'
        wanted = r'(?:a|b)'
        result = retools.consolidate(regex)
        self.assertEqual(result, wanted)

    def test_danger_unpack(self):
        regex = r'(abc)'
        wanted = r'abc'
        result = retools.danger_unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'(?:abc)'
        wanted = r'abc'
        result = retools.danger_unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'(?P<xyz>abc)'
        wanted = r'abc'
        result = retools.danger_unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'[abc]'
        wanted = r'[abc]'
        result = retools.danger_unpack(regex)
        self.assertEqual(result, wanted)

    def test_unpack(self):
        regex = r'(abc)'
        wanted = r'abc'
        result = retools.unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'(?:abc)'
        wanted = r'abc'
        result = retools.unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'(?P<xyz>abc)'
        wanted = r'(?P<xyz>abc)'
        result = retools.unpack(regex)
        self.assertEqual(result, wanted)

        regex = r'[abc]'
        wanted = r'[abc]'
        result = retools.unpack(regex)
        self.assertEqual(result, wanted)

    def test_parallel(self):
        result = retools.parallel([r'abc', r'def'])
        wanted = r'abc|def'
        self.assertEqual(result, wanted)

        result = retools.parallel([r'abc', r'd|ef'])
        wanted = 'abc|d|ef'
        self.assertEqual(result, wanted)

        result = retools.parallel([r'abc', r'(d|ef)'])
        wanted = 'abc|d|ef'
        self.assertEqual(result, wanted)

    def test_nocatch(self):
        regex = r'a|b'
        wanted = r'(?:a|b)'
        result = retools.nocatch(regex)
        self.assertEqual(result, wanted)

        regex = r'(a|b)'
        wanted = r'(?:a|b)'
        result = retools.nocatch(regex)
        self.assertEqual(result, wanted)

        regex = r'(?P<x>ab)'
        wanted = r'(?:ab)'
        result = retools.nocatch(regex)
        self.assertEqual(result, wanted)

        regex = r'[ab]'
        wanted = r'[ab]'
        result = retools.nocatch(regex)
        self.assertEqual(result, wanted)

    def test_concat(self):
        regex_1 = r'a|b'
        regex_2 = r'(c|de)'
        regex_3 = r'[fg]'

        result_12 = retools.concat([regex_1, regex_2])
        wanted_12 = r'(a|b)(c|de)'
        self.assertEqual(result_12, wanted_12)

        result_13 = retools.concat([regex_1, regex_3])
        wanted_13 = r'(a|b)[fg]'
        self.assertEqual(result_13, wanted_13)

        result_123 = retools.concat([regex_1, regex_2, regex_3])
        wanted_123 = r'(a|b)(c|de)[fg]'
        self.assertEqual(result_123, wanted_123)

    def test_nocatchall(self):
        regex = r'abc'
        wanted = r'abc'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

        regex = r'(abc)'
        wanted = r'(?:abc)'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

        regex = r'(?:abc)'
        wanted = r'(?:abc)'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

        regex = r'(?P<xyz>abc)'
        wanted = r'(?:abc)'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

        regex = r'(abc(?P<xyz>def))'
        wanted = r'(?:abc(?:def))'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

        regex = r'\(abc\)'
        wanted = r'\(abc\)'
        result = retools.nocatchall(regex)
        self.assertEqual(result, wanted)

    def test_option(self):
        regex = r'abc'
        wanted = r'(?:abc)?'
        result = retools.option(regex)
        self.assertEqual(result, wanted)

        regex = r'(abc)'
        wanted = r'(?:abc)?'
        result = retools.option(regex)
        self.assertEqual(result, wanted)

        regex = r'[abc]'
        wanted = r'[abc]?'
        result = retools.option(regex)
        self.assertEqual(result, wanted)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import re
from itertools import chain, combinations, cycle, islice
from collections import namedtuple

def sent_tokenize(context):
    """
    Cut the given context into sentences.
    Avoid a linebreak in between paried symbols, float numbers, and some abbrs.
    Nothing will be discard after sent_tokeinze, simply ''.join(sents) will get the original context.
    Evey whitespace, tab, linebreak will be kept.

    >>> context = "I love you. Please don't leave."
    >>> sent_tokenize(context)
    ["I love you. ", "Please don't leave."]
    """

    # Define the regular expression
    paired_symbols = [("(", ")"),
                  ("[", "]"),
                  ("{", "}")]
    paired_patterns = ["%s.*?%s" % (re.escape(lt), re.escape(rt)) for lt, rt in paired_symbols]
    number_pattern = ['\d+\.\d+']
    arr_pattern = ['(?: \w\.){2,3}|(?:\A|\s)(?:\w\.){2,3}|[A-Z]\. [a-z]|\svs\. |et al\.|Fig\. \d|approx\.|(?:Prof|Dr)\. (?:[A-Z]\.)?']

    # Find the string which matches the above pattern, and remove than from the context, to get a stem string
    escape_re = re.compile("|".join(paired_patterns + number_pattern + arr_pattern))
    escapes = escape_re.findall(context)
    escaped_stem = escape_re.sub('{}', context)

    escaped_escaped_stem = escaped_stem.replace('{','{{').replace('}', '}}')

    # Find the linebreaks
    sent_re = re.compile(r'([A-Z0-9]..+?(?:[.!?]\s|[\n$]))')
    linebreaks = sent_re.findall(escaped_escaped_stem)
    sent_stem = sent_re.sub(r'\1###linebreak###', escaped_escaped_stem)
    recovered_sent_stem = sent_stem.replace('{{}}', '{}')
    result = recovered_sent_stem.format(*escapes)
    return [r for r in result.split('###linebreak###') if r is not '']

def sent_count(context):
    """
    Return the sentence counts for given context

    >>> context = "I love you. Please don't leave."
    >>> sent_count(context)
    2
    """
    return len(sent_tokenize(context))

def clause_tokenize(sentence):
    """
    Split on comma or parenthesis, if there are more then three words for each clause

    >>> context = 'While I was walking home, this bird fell down in front of me.'
    >>> clause_tokenize(context)
    ['While I was walking home,', ' this bird fell down in front of me.']

    """
    clause_re = re.compile(r'((?:\S+\s){2,}\S+,|(?:\S+\s){3,}(?=\((?:\S+\s){2,}\S+\)))')
    clause_stem = clause_re.sub(r'\1###clausebreak###', sentence)
    return [c for c in clause_stem.split('###clausebreak###') if c != '']

def word_tokenize(sentence):
    """
    A generator which yields tokens based on the given sentence without deleting anything.

    >>> context = "I love you. Please don't leave."
    >>> list(word_tokenize(context))
    ['I', ' ', 'love', ' ', 'you', '.', ' ', 'Please', ' ', 'don', "'", 't', ' ', 'leave', '.']

    """
    date_pattern = r'\d\d(\d\d)?[\\-]\d\d[\\-]\d\d(\d\d)?'
    number_pattern = r'[\+-]?(\d+\.\d+|\d{1,3},(\d{3},)*\d{3}|\d+)'
    arr_pattern = r'(?: \w\.){2,3}|(?:\A|\s)(?:\w\.){2,3}|[A-Z]\. [a-z]'
    word_pattern = r'[\w]+'
    non_space_pattern = r'[{}]|\w'.format(re.escape('!"#$%&()*,./:;<=>?@[\]^_-`{|}~'))
    space_pattern = r'\s'
    anything_pattern = r'.'
    patterns = [date_pattern, number_pattern, arr_pattern, word_pattern, non_space_pattern, space_pattern, anything_pattern]
    big_pattern = r'|'.join([('(' + pattern + ')') for pattern in patterns])
    for match in re.finditer(big_pattern, sentence):
        yield match.group(0)

def slim_stem(token):
    """
    A very simple stemmer, for entity of GO stemming.

    >>> token = 'interaction'
    >>> slim_stem(token)
    'interact'

    """
    target_sulfixs = ['ic', 'tic', 'e', 'ive', 'ing', 'ical', 'nal', 'al', 'ism', 'ion', 'ation', 'ar', 'sis', 'us', 'ment']
    for sulfix in sorted(target_sulfixs, key=len, reverse=True):
        if token.endswith(sulfix):
            token = token[0:-len(sulfix)]
            break
    if token.endswith('ll'):
        token = token[:-1]
    return token

def powerset(iterable):
    """
    powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)
    """
    s = list(iterable)
    return chain.from_iterable(combinations(s, r) for r in range(len(s)+1))

def ngram(n, iter_tokens):
    """
    Return a generator of n-gram from an iterable
    """
    z = len(iter_tokens)
    return (iter_tokens[i:i+n] for i in range(z-n+1))

def power_ngram(iter_tokens):
    """
    Generate unigram, bigram, trigram ... and the max-gram,
    different from powerset(), this function will not generate skipped combinations such as (1,3)
    """
    return chain.from_iterable(ngram(j, iter_tokens) for j in range(1, len(iter_tokens) + 1))

def count_start(tokenizer):
    """
    A decorator which wrap the given tokenizer to yield (token, start).
    Notice! the decorated tokenizer must take a int arguments stands for the start position of the input context/sentence
    
    >>> tokenizer = lambda sentence: sentence.split(' ')
    >>> tokenizer('The quick brown fox jumps over the lazy dog')
    ['The', 'quick', 'brown', 'fox', 'jumps', 'over', 'the',
    'lazy', 'dog']
    >>> tokenizer = count_start(tokenizer)
    >>> tokenizer('The quick brown fox jumps over the lazy dog', 0)
    
    ('The', 0)
    ('quick', 4)
    ...
    """
    def wrapper(context, base):
        tokens = list(tokenizer(context))
        flag = 0
        for token in tokens:
            start = context.index(token, flag)
            flag = start + len(token)
            yield (token, base + start)
    
    return wrapper
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

from collections import OrderedDict
import re
import string

def lexical(token):
    """
    Extract lexical features from given token
    There are 3 kinds of lexical features, take 'Hello' as an example:

    1. lowercase: 'hello'
    2. first4: 'hell'
    3. last4: 'ello'
    """
    lowercase = token.lower()
    first4 = lowercase[:4]
    last4 = lowercase[-4:]
    return OrderedDict([
            ('lowercase', lowercase), 
            ('first4', first4),
            ('last4', last4)
            ])
            
def _char_shape(char):
    if char in string.ascii_uppercase:
        return 'A'
    if char in string.ascii_lowercase:
        return 'a'
    if char in string.digits:
        return '0'
    else:
        return char

def _shape(token):
    return ''.join([_char_shape(char) for char in token])

def _contains_a_letter(token):
    regex = r'[A-Za-z]'
    if re.search(regex, token):
        return True
    else:
        return False
    
def _contains_a_capital(token):
    regex = r'[A-Z]'
    if re.search(regex, token):
        return True
    else:
        return False

def _begins_with_capital(token):
    return _char_shape(token[0]) == 'A'

def _all_capital(token):
    regex = r'^[A-Z]+$'
    if re.match(regex, token):
        return True
    else:
        return False

def _contains_a_digit(token):
    regex = r'\d'
    if re.search(regex, token):
        return True
    else:
        return False

def _all_digit(token):
    regex = r'^\d+$'
    if re.match(regex, token):
        return True
    else:
        return False

def _contains_a_punctuation(token):
    return len(set(string.punctuation) & set(token)) > 0

def _consists_letters_n_digits(token):
    shape = _shape(token)
    return set(shape.lower()) == set('a0')

def _consists_digits_n_punctuations(token):
    shape = _shape(token)
    lower_shape = shape.lower()
    return set(lower_shape) <= set(string.punctuation+'0') and len(lower_shape) >= 2
    
def orthographic(token):
    """
    Extract orthographic features from a given token
    
    There are 11 kinds of orthographic features, take 'Windows10' as an example:

    1. shape: 'Aaaaaaa00'
    2. length: 9
    3. contains_a_letter: True
    4. contains_a_capital: True
    5. begins_with_capital: True
    6. all_capital: False
    7. contains_a_digit: True
    8. all_digit: False
    9. contains_a_punctuation: False
    10. consists_letters_n_digits: True
    11. consists_digits_n_punctuations: False
    """

    return OrderedDict([
                    ('shape', _shape(token)), 
                    ('length', len(token)), 
                    ('contains_a_letter', _contains_a_letter(token)), 
                    ('contains_a_capital', _contains_a_capital(token)),
                    ('begins_with_capital', _begins_with_capital(token)), 
                    ('all_capital', _all_capital(token)), 
                    ('contains_a_digit', _contains_a_digit(token)),
                    ('all_digit', _all_digit(token)),
                    ('contains_a_punctuation', _contains_a_punctuation(token)),
                    ('consists_letters_n_digits', _consists_letters_n_digits(token)),
                    ('consists_digits_n_punctuations', _consists_digits_n_punctuations(token)),
                   ])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

"""
test_txttk
----------------------------------

Tests for `txttk` module.
"""

import unittest
import txttk

class TestTxttk(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import unittest
from txttk import nlptools

class NlptoolsTestCase(unittest.TestCase):
    def setUp(self):
        self.sentences = """A common problem faced by biocurators when using text mining systems is that these are difficult to use or do not provide an output that can be directly exploited by biocurators during their literature curation process. 
In this respect, the BioCreative Interactive {Text Mining} (IAT) task has served as a great means to observe the approaches, standards and functionalities used by state-of-the-art text mining systems with potential applications in the biocuration domain. 
The IAT task also provides a means for biocurators to be directly involved in the testing of text mining systems. 
For the upcoming BioCreative V, seven teams have submitted a text mining/NLP system targeted to a specific biocuration task. 
These systems will be formally evaluated by users, but not competitively.""".split('\n')
        self.context = "".join(self.sentences)

    #def test_roundrobin(self):
    #    self.assertEqual(list(nlptools.roundrobin('ABC', 'D', 'EF')), ['A', 'D', 'E', 'B', 'F', 'C'])

    def test_sent_tokenize(self):
        self.assertEqual(nlptools.sent_tokenize(self.context), self.sentences)

    def test_sent_tokenize_intergration(self):
        self.assertEqual(''.join(nlptools.sent_tokenize(self.context)), self.context)

    def test_sent_count(self):
        self.assertEqual(nlptools.sent_count(self.context), 5)

    def test_clause_tokenize(self):
        sentence = self.sentences[1]
        wanted = ['In this respect, the BioCreative Interactive {Text Mining} (IAT) task has served as a great means to observe the approaches,',
                  ' standards and functionalities used by state-of-the-art text mining systems with potential applications in the biocuration domain. ']
        self.assertEqual(nlptools.clause_tokenize(sentence), wanted)

    def test_clause_tokenize_intergration(self):
        self.assertEqual(''.join(nlptools.clause_tokenize(self.sentences[1])), self.sentences[1])

    def test_word_tokenzie(self):
        sentence = 'A 2.1 cm tumor (right tongue) noted on 2013-11-11.'
        wanted = ['A', ' ', '2.1', ' ', 'cm', ' ', 'tumor', ' ', '(', 'right', ' ', 'tongue', ')', ' ', 'noted', ' ', 'on', ' ', '2013-11-11', '.']
        self.assertEqual(list(nlptools.word_tokenize(sentence)), wanted)
    
    def test_word_tokenzie2(self):
        sentence = '-999 1,234,000 3.1415'
        wanted = ['-999', ' ', '1,234,000', ' ', '3.1415']
        self.assertEqual(list(nlptools.word_tokenize(sentence)), wanted)

    def test_word_tokenize_intergration(self):
        for sent in self.sentences:
          self.assertEqual(''.join(list(nlptools.word_tokenize(sent))), sent)

    def test_slim_stem(self):
        """['ic', 'tic', 'e', 'ive', 'ing', 'ical', 'nal', 'al', 'ism', 'ion', 'ation', 'ar', 'sis', 'us', 'ment']"""
        word2target = [('magic', 'mag'),
                       ('automatic', 'automa'),
                       ('life', 'lif'),
                       ('active', 'act'),
                       ('hiking', 'hik'),
                       ('medical', 'med'),
                       ('national', 'natio'),
                       ('animal', 'anim'),
                       ('Parkinsonism', 'Parkinson'),
                       ('mission', 'miss'),
                       ('medication', 'medic'),
                       ('radar', 'rad'),
                       ('dialysis', 'dialy'),
                       ('status', 'stat'),
                       ('measurement', 'measure'),
                       ('signalling', 'signal')]
        for word, target in word2target:
            self.assertEqual(nlptools.slim_stem(word), target)

    def test_powerset(self):
        wanted = [(), (1,), (2,), (3,), (1, 2), (1, 3), (2, 3), (1, 2, 3)]
        self.assertEqual(list(nlptools.powerset([1,2,3])), wanted)

    def test_ngram(self):
        tokens = 'hello world kitty'.split(' ')

        self.assertEqual(list(nlptools.ngram(2, tokens)), [['hello', 'world'], ['world', 'kitty']])
        self.assertEqual(list(nlptools.ngram(3, tokens)), [tokens])
        self.assertEqual(list(nlptools.ngram(4, tokens)), [])

    def test_power_ngram(self):
        tokens = 'hello world kitty'.split(' ')

        self.assertEqual(list(nlptools.power_ngram(tokens)), \
            [['hello'], ['world'], ['kitty'], ['hello', 'world'], ['world', 'kitty'], ['hello', 'world', 'kitty']])

    def test_count_start(self):
        tokenizer = lambda sentence: sentence.split(' ')
        sentence = 'The quick brown'
        tokenizer = nlptools.count_start(tokenizer)
        result = [start for token, start in tokenizer(sentence, 0)]
        wanted = [0, 4, 10]
        self.assertEqual(result, wanted)
        
        sentence = 'jumps over the'
        result = [start for token, start in tokenizer(sentence, 16)]
        wanted = [16, 22, 27]
        self.assertEqual(result, wanted)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

from setuptools import setup

with open('README.rst') as f:
    readme = f.read()

with open('VERSION') as f:
    version = f.read().strip()

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist bdist_wheel upload")
    sys.exit()

requirements = [
    # TODO: put package requirements here
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='txttk',
    version=version,
    description="Text Toolkit: manipulate text & regular expression",
    long_description=readme,
    author="Chia-Jung, Yang",
    author_email='jeroyang@gmail.com',
    url='https://github.com/jeroyang/txttk',
    packages=[
        'txttk',
    ],
    package_dir={'txttk':
                 'txttk'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='txttk',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import os
from txttk import nlptools, retools, feature

__author__ = 'Chia-Jung, Yang'
__email__ = 'jeroyang@gmail.com'

version_path = os.path.join(os.path.dirname(__file__), '..', 'VERSION')
with open(version_path) as f:
    __version__ = f.read().strip()

__all__ = ['feature', 'nlptools', 'retools']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division,
                        print_function, unicode_literals)
from builtins import *

import unittest
from collections import OrderedDict
from txttk import feature

class FeatureTestCase(unittest.TestCase):
    def test_lexical(self):
        token = 'Hello'
        result = feature.lexical(token)
        wanted = OrderedDict([
                ('lowercase', 'hello'),
                ('first4', 'hell'),
                ('last4', 'ello')
                ])
        self.assertEqual(result, wanted)
    
    def test_char_shape(self):
        char = 'X'
        wanted = 'A'
        result = feature._char_shape(char)
        self.assertEqual(result, wanted)
        
        char = '8'
        wanted = '0'
        result = feature._char_shape(char)
        self.assertEqual(result, wanted)
        
        char = '-'
        wanted = '-'
        result = feature._char_shape(char)
        self.assertEqual(result, wanted)
        
    def test_shape(self):
        token = 'Abc-987'
        wanted = 'Aaa-000'
        result = feature._shape(token)
        self.assertEqual(result, wanted)
        
    def test_contains_a_letter(self):
        token = 'a123'
        result = feature._contains_a_letter(token)
        self.assertTrue(result)
        
        token = '1234'
        result = feature._contains_a_letter(token)
        self.assertFalse(result)
    
    def test_contains_a_capital(self):
        token = 'A001'
        result = feature._contains_a_capital(token)
        self.assertTrue(result)
        
        token = 'abcd'
        result = feature._contains_a_capital(token)
        self.assertFalse(result)
    
    def test_all_capital(self):
        token = 'XYZ'
        result = feature._all_capital(token)
        self.assertTrue(result)
        
        token = 'XyZ'
        result = feature._all_capital(token)
        self.assertFalse(result)
        
    def test_contains_a_digit(self):
        token = 'A001'
        result = feature._contains_a_digit(token)
        self.assertTrue(result)
        
        token = 'ab-xy'
        result = feature._contains_a_digit(token)
        self.assertFalse(result)
        
    def test_all_digit(self):
        token = '1238'
        result = feature._all_digit(token)
        self.assertTrue(result)
        
        token = 'A123'
        result = feature._all_digit(token)
        self.assertFalse(result)
        
    def test_contains_a_punctuation(self):
        token = 'abc*'
        result = feature._contains_a_punctuation(token)
        self.assertTrue(result)
        
        token = 'a123'
        result = feature._contains_a_punctuation(token)
        self.assertFalse(result)
        
    def test_consists_letters_n_digits(self):
        token = 'Ax123'
        result = feature._consists_letters_n_digits(token)
        self.assertTrue(result)
    
        token = '1-2-3'
        result = feature._consists_letters_n_digits(token)
        self.assertFalse(result)
    
    def test_consists_digits_n_punctuations(self):
        token = '123-123-123'
        result = feature._consists_digits_n_punctuations(token)
        self.assertTrue(result)
        
        token = 'ax123-422'
        result = feature._consists_digits_n_punctuations(token)
        self.assertFalse(result)
    
    def test_orthographic(self):
        token = 'JeroYang123'
        result = feature.orthographic(token)
        wanted = ['AaaaAaaa000',
                  11,
                  True,
                  True,
                  True,
                  False,
                  True,
                  False,
                  False,
                  True,
                  False
                  ]
        self.assertEqual(list(result.values()), wanted)